@@ -10,6 +10,7 @@ import argparse
 from knack.arguments import ArgumentsContext, CLIArgumentType
 
 from azdev.completer import get_test_completion
+from azdev.operations.linter import linter_severity_choices
 
 
 class Flag(object):
@@ -36,6 +37,7 @@ def load_arguments(self, _):
         c.argument('cli_path', options_list=['--cli', '-c'], nargs='?', const=Flag, help="Path to an existing Azure CLI repo. Omit value to search for the repo or use special value 'EDGE' to install the latest developer edge build.")
         c.argument('ext_repo_path', options_list=['--repo', '-r'], nargs='+', help='Space-separated list of paths to existing Azure CLI extensions repos.')
         c.argument('ext', options_list=['--ext', '-e'], nargs='+', help="Space-separated list of extensions to install initially. Use '*' to install all extensions.")
+        c.argument('deps', options_list=['--deps-from', '-d'], choices=['requirements.txt', 'setup.py'], default='requirements.txt', help="Choose the file to resolve dependencies.")
 
     with ArgumentsContext(self, 'test') as c:
         c.argument('discover', options_list='--discover', action='store_true', help='Build an index of test names so that you don\'t need to specify fully qualified test paths.')
@@ -84,11 +86,25 @@ def load_arguments(self, _):
         c.argument('include_whl_extensions',
                    action='store_true',
                    help="Allow running linter on extensions installed by `az extension add`.")
+        c.argument('save_global_exclusion',
+                   action='store_true',
+                   options_list=['--save', '-s'],
+                   help="Allow saving global exclusion. It would take effect when modules is CLI or EXT.",
+                   deprecate_info=c.deprecate(hide=True))
+        c.argument('min_severity', choices=linter_severity_choices(),
+                   help='The minimum severity level to run the linter on. '
+                        'For example, specifying "medium" runs linter rules that have "high" or "medium" severity. '
+                        'However, specifying "low" runs the linter on every rule, regardless of severity. '
+                        'Defaults to "high".')
     # endregion
 
     with ArgumentsContext(self, 'perf') as c:
         c.argument('runs', type=int, help='Number of runs to average performance over.')
 
+    with ArgumentsContext(self, 'perf benchmark') as c:
+        c.positional('commands', nargs="+", help="Command prefix to run benchmark")
+        c.argument('top', type=int, help='Show N slowest commands. 0 for all.')
+
     with ArgumentsContext(self, 'extension') as c:
         c.argument('dist_dir', help='Name of a directory in which to save the resulting WHL files.')
 
